@@ -16,6 +16,7 @@ import types
 import logging
 import tempfile
 import threading
+import inspect
 
 import wx
 import wx.lib
@@ -68,7 +69,10 @@ class WbApp(wx.App):
 
         self.setupLogging()
 
-        self.prefs = wb_preferences.Preferences( self )
+        self.prefs = wb_preferences.Preferences(
+                self,
+                wb_platform_specific.getPreferencesFilename(),
+                wb_platform_specific.getOldPreferencesFilename() )
 
         self.lock_ui = 0
         self.need_activate_app_action = False
@@ -209,6 +213,18 @@ class WbApp(wx.App):
     def getPasteData( self ):
         return self.__paste_data
 
+    def diffFiles( self, file_left, title_left, file_right, title_right ):
+        diff_frame = wb_diff_frame.DiffFrame(
+            self, self.frame,
+            file_left, title_left,
+            file_right, title_right )
+        # only show if the files could be read
+        if diff_frame.isOk():
+            diff_frame.showAllFolds( False )
+            diff_frame.Show( True )
+
+            self.all_diff_frames.append( diff_frame )
+
     def DiffDone( self, diff_frame ):
         self.all_diff_frames.remove( diff_frame )
 
@@ -335,6 +351,22 @@ class WbApp(wx.App):
             self.log.exception( 'OnAppCallBack<%s.%s>\n' %
                 (event.callback.__module__, event.callback.__name__ ) )
 
+    def debugShowCallers( self, depth ):
+        if not self.__debug:
+            return
+
+        stack = inspect.stack()
+        for index in range( 1, depth+1 ):
+            if index >= len(stack):
+                break
+
+            caller = stack[ index ]
+            filename = os.path.basename( caller[1] )
+            self.log.debug( 'File: %s:%d, Function: %s' % (filename, caller[2], caller[3]) )
+            del caller
+
+        del stack
+
 class AppBackgroundFunction:
     def __init__( self, app, function, args ):
         self.app = app
