@@ -27,7 +27,7 @@ class DialogBuildingBlock(wx.Dialog):
         self.add_log_message_field = False
         self.add_force_field = False
 
-    def initControl( self ):
+    def initControls( self ):
         self.v_sizer = wx.BoxSizer( wx.VERTICAL )
         self.h_sizer = wx.BoxSizer( wx.HORIZONTAL )
 
@@ -39,7 +39,7 @@ class DialogBuildingBlock(wx.Dialog):
             self.filename_list.InsertColumn( 1, "Filename" )
             self.filename_list.SetColumnWidth( 1, 1000 )
 
-            for index, filename in enumerate( self.all_filenames ):
+            for index, _ in enumerate( self.all_filenames ):
                 self.filename_list.InsertStringItem( index, self.all_filenames[index][0] )
                 self.filename_list.SetStringItem( index, 1, self.all_filenames[index][1] )
 
@@ -105,7 +105,7 @@ class ConfirmAction(DialogBuildingBlock):
         self.add_filename_list_field = True
         self.add_force_field = force_field
 
-        self.initControl()
+        self.initControls()
 
 class LogMessage(DialogBuildingBlock):
     def __init__( self, parent, title, all_filenames, message_filename=None, force_field=False ):
@@ -121,10 +121,10 @@ class LogMessage(DialogBuildingBlock):
         if self.message_filename is not None:
             try:
                 self.last_log_message_text = file( self.message_filename, 'r' ).read().decode('utf-8').strip()
-            except (IOError,OSError):
+            except EnvironmentError:
                 self.last_log_message_text = ''
 
-        self.initControl()
+        self.initControls()
 
     def initExtraButtons( self ):
         if self.last_log_message_text is not None:
@@ -517,7 +517,8 @@ class NewFile(wx.Dialog):
 
     def getTemplateFilename( self ):
         index = self.template_list.GetCurrentSelection()
-        if len(self.template_file_list) > index:
+        print 'getTemplateFilename',len(self.template_file_list),index
+        if index >= 0 and len(self.template_file_list) > index:
             return self.template_file_list[ index ] + self.template_suffix
         else:
             return None
