@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2007 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -29,17 +29,6 @@ import wb_bookmarks_dialogs
 
 import wb_config
 
-# point size and face need to choosen for platform
-if wx.Platform == '__WXMSW__':
-    face = 'Courier New'
-    point_size = 8
-elif wx.Platform == '__WXMAC__':
-    face = 'Monaco'
-    point_size = 12
-else:
-    face = 'Courier'
-    point_size = 12
-
 class WbFrame(wx.Frame):
     status_general = 0
     status_search = 0    # use the general area
@@ -134,11 +123,18 @@ class WbFrame(wx.Frame):
         self.menu_view.AppendCheckItem( wb_ids.id_View_UncontrolledFiles, "Show &Uncontrolled files", "Show Uncontrolled files" )
         self.menu_view.AppendCheckItem( wb_ids.id_View_IgnoredFiles, "Show &Ignored files", "Show ignored files" )
         self.menu_view.AppendSeparator()
+        self.menu_view.AppendCheckItem( wb_ids.id_View_Diff_WbDiff, 'Use WorkBench Diff' )
+        self.menu_view.AppendCheckItem( wb_ids.id_View_Diff_ExtGuiDiff, 'Use External GUI Diff' )
+        self.menu_view.AppendCheckItem( wb_ids.id_View_Diff_ExtTextDiff, 'Use External Text Diff' )
+        self.menu_view.AppendCheckItem( wb_ids.id_View_Diff_SvnDiff, 'Use SVN Diff' )
+        self.menu_view.AppendSeparator()
         self.menu_view.AppendCheckItem( wb_ids.id_View_Recursive, "Show &Recursive files", "Show recursive files" )
         self.menu_view.AppendSeparator()
         self.menu_view.Append( wb_ids.id_View_Refresh, "&Refresh\tF5", "Refresh display" )
         self.menu_view.AppendCheckItem( wb_ids.id_View_AutoRefresh, "&Automatic Refresh", "Automatic refresh" )
 
+
+
         self.all_bookmark_ids = {}
         self.all_bookmark_top_level_menu_ids = []
         self.all_bookmark_folders = {}
@@ -310,6 +306,14 @@ class WbFrame(wx.Frame):
             wx.EVT_MENU( event_source, wb_ids.id_SP_EditPaste, self.app.eventWrapper( self.OnSpEditPaste ) )
             wx.EVT_UPDATE_UI( event_source, wb_ids.id_SP_EditPaste, self.app.eventWrapper( self.OnUpdateUiSpEditPaste ) )
 
+            wx.EVT_MENU( event_source, wb_ids.id_View_Diff_WbDiff, self.app.eventWrapper( self.OnViewDiffWbDiff ) )
+            wx.EVT_UPDATE_UI( event_source, wb_ids.id_View_Diff_WbDiff, self.app.eventWrapper( self.OnUpdateUiViewDiffWbDiff ) )
+            wx.EVT_MENU( event_source, wb_ids.id_View_Diff_ExtGuiDiff, self.app.eventWrapper( self.OnViewDiffExtGuiDiff ) )
+            wx.EVT_UPDATE_UI( event_source, wb_ids.id_View_Diff_ExtGuiDiff, self.app.eventWrapper( self.OnUpdateUiViewDiffExtGuiDiff ) )
+            wx.EVT_MENU( event_source, wb_ids.id_View_Diff_ExtTextDiff, self.app.eventWrapper( self.OnViewDiffExtTextDiff ) )
+            wx.EVT_UPDATE_UI( event_source, wb_ids.id_View_Diff_ExtTextDiff, self.app.eventWrapper( self.OnUpdateUiViewDiffExtTextDiff ) )
+            wx.EVT_MENU( event_source, wb_ids.id_View_Diff_SvnDiff, self.app.eventWrapper( self.OnViewDiffSvnDiff ) )
+            wx.EVT_UPDATE_UI( event_source, wb_ids.id_View_Diff_SvnDiff, self.app.eventWrapper( self.OnUpdateUiViewDiffSvnDiff ) )
 
         wx.EVT_MENU( self, wb_ids.id_File_Edit, try_wrapper( self.OnFileEdit ) )
         wx.EVT_UPDATE_UI( self, wb_ids.id_File_Edit, try_wrapper( self.OnUpdateUiFileEdit ) )
@@ -412,17 +416,27 @@ class WbFrame(wx.Frame):
         self.list_panel.list_ctrl.SetFocus()
 
     def setEventHandler( self, handler ):
-        self.app.log.debug( 'setEventHandler from %r' % self.event_handler )
-        self.app.log.debug( 'setEventHandler   to %r' % handler )
+        if wb_config.debug_selection:
+            print 'ZF: setEventHandler from %r' % self.event_handler
+            print 'ZF: setEventHandler   to %r' % handler
+            import wb_debug
+            wb_debug.printStack( '     ' )
+
+        self.app.debugShowCallers( 20 )
+
         if self.event_handler is not handler:
             self.event_handler = handler
             self.clearUpdateUiState()
 
     def clearEventHandler( self ):
         self.app.log.debug( 'clearEventHandler from %r to None' % self.event_handler )
+        if wb_config.debug_selection: print 'ZF: clearEventHandler from %r to None' % self.event_handler
         self.event_handler = None
         self.clearUpdateUiState()
 
+    def isEventHandler( self, handler ):
+        return self.event_handler is handler
+
     # Status bar settings
     def setStatus( self, text ):
         self.GetStatusBar().SetStatusText( text, WbFrame.status_general )
@@ -456,7 +470,7 @@ class WbFrame(wx.Frame):
                 '\n' + wb_source_control_providers.getProviderAboutStrings() +
                 '\nWxPython %d.%d.%d.%d %s' % wx.VERSION +
                 '\nPython %d.%d.%d %s %d\n' % sys.version_info +
-                '\nCopyright Barry Scott (c) 2003-2006. All rights reserved'
+                '\nCopyright Barry Scott (c) 2003-2007. All rights reserved'
                 )
         wx.LogMessage( str_message )
 
@@ -562,10 +576,42 @@ class WbFrame(wx.Frame):
         view_prefs = self.app.prefs.getView()
         event.Check( view_prefs.view_recursive )
 
+    def OnViewDiffWbDiff( self, event ):
+        self.app.prefs.getDiffTool().diff_tool_mode = 'built-in'
+        self.app.prefs.writePreferences()
+
+    def OnUpdateUiViewDiffWbDiff( self, event ):
+        event.Check( self.app.prefs.getDiffTool().diff_tool_mode == 'built-in' )
+
+    def OnViewDiffExtGuiDiff( self, event ):
+        self.app.prefs.getDiffTool().diff_tool_mode = 'external-gui-diff'
+        self.app.prefs.writePreferences()
+
+    def OnUpdateUiViewDiffExtGuiDiff( self, event ):
+        event.Enable( self.app.prefs.getDiffTool().gui_diff_tool != '' )
+        event.Check( self.app.prefs.getDiffTool().diff_tool_mode == 'external-gui-diff' )
+
+    def OnViewDiffExtTextDiff( self, event ):
+        self.app.prefs.getDiffTool().diff_tool_mode = 'external-shell-diff'
+        self.app.prefs.writePreferences()
+
+    def OnUpdateUiViewDiffExtTextDiff( self, event ):
+        event.Enable( self.app.prefs.getDiffTool().shell_diff_tool != '' )
+        event.Check( self.app.prefs.getDiffTool().diff_tool_mode == 'external-shell-diff' )
+
+    def OnViewDiffSvnDiff( self, event ):
+        self.app.prefs.getDiffTool().diff_tool_mode = 'svn-diff'
+        self.app.prefs.writePreferences()
+
+    def OnUpdateUiViewDiffSvnDiff( self, event ):
+        event.Check( self.app.prefs.getDiffTool().diff_tool_mode == 'svn-diff' )
+
     def OnRefresh( self, event ):
+        self.app.log.debug( 'OnRefresh()' )
         self.refreshFrame()
 
     def refreshFrame( self ):
+        self.app.log.debug( 'WbFrame.refreshFrame()' )
         # tell the tree to refresh it will tell the list
         self.tree_panel.refreshTree()
 
@@ -705,7 +751,6 @@ class WbFrame(wx.Frame):
         self.getUpdateUiState()
         event.Enable( self.ui_state_tree.file_exists )
 
-
     def OnSpEditCopy( self, event ):
         return self.Sp_Dispatch( 'OnSpEditCopy' )
 
@@ -743,7 +788,7 @@ class WbFrame(wx.Frame):
         if self.ui_state_focus.need_checkout:
             event.Enable( False )
         else:
-            event.Enable( self.ui_state_tree.versioned )
+            event.Enable( self.ui_state_focus.versioned and (not self.ui_state_focus.is_folder) )
 
     def OnSpCheckin( self, event ):
         return self.Sp_Dispatch( 'OnSpCheckin' )
@@ -853,6 +898,8 @@ class WbFrame(wx.Frame):
 
     def OnUpdateUiSpHistory( self, event ):
         self.getUpdateUiState()
+        if wb_config.debug_selection_update: print 'ZF: OnUpdateUiSpHistory versioned %r handler %r' % (
+                                                self.ui_state_focus.versioned, self.event_handler)
         if self.ui_state_focus.need_checkout:
             event.Enable( False )
         else:
@@ -873,17 +920,18 @@ class WbFrame(wx.Frame):
 
     def OnUpdateUiSpLock( self, event ):
         self.getUpdateUiState()
-        event.Enable( self.ui_state_tree.file_exists )
+        event.Enable( (not self.ui_state_focus.is_folder) and self.ui_state_focus.file_exists )
 
     def OnSpMkdir( self, event ):
-        return self.Sp_Dispatch( 'OnSpMkdir' )
+        # always forward to the tree to handle
+        return self.tree_panel.OnSpMkdir()
 
     def OnUpdateUiSpMkdir( self, event ):
         self.getUpdateUiState()
         event.Enable( self.ui_state_tree.file_exists )
 
     def OnSpNewFile( self, event ):
-        return self.Sp_Dispatch( 'OnSpNewFile' )
+        return self.tree_panel.OnSpNewFile()
 
     def OnUpdateUiSpNewFile( self, event ):
         self.getUpdateUiState()
@@ -959,7 +1007,7 @@ class WbFrame(wx.Frame):
 
     def OnUpdateUiSpUnlock( self, event ):
         self.getUpdateUiState()
-        event.Enable( self.ui_state_tree.file_exists )
+        event.Enable( (not self.ui_state_focus.is_folder) and self.ui_state_focus.file_exists )
 
     def OnSpUpdate( self, event ):
         return self.Sp_Dispatch( 'OnSpUpdate' )
@@ -999,42 +1047,44 @@ class WbFrame(wx.Frame):
             return fn()
 
     def getUpdateUiState( self ):
+        all_debug_messages = []
         if self.ui_state_tree is None:
-            if wb_config.debug_selection: print 'Z: getUpdateUiState() tree'
             self.ui_state_tree = self.tree_panel.getUpdateUiState()
             if self.ui_state_tree is None:
-                self.ui_state_tree = wb_tree_panel.TreeState()
-            if wb_config.debug_selection: print 'Z: getUpdateUiState() tree need_checkout', self.ui_state_tree.need_checkout
+                self.ui_state_tree = wb_tree_panel.TreeState( True )
+            all_debug_messages.append( 'tree place_holder %s' % self.ui_state_tree.place_holder )
 
         if self.ui_state_list is None:
-            if wb_config.debug_selection: print 'Z: getUpdateUiState() list'
             self.ui_state_list = self.list_panel.getUpdateUiState()
             if self.ui_state_list is None:
-                self.ui_state_list = wb_list_panel_common.ListItemState()
-            if wb_config.debug_selection: print 'Z: getUpdateUiState() list need_checkout', self.ui_state_list.need_checkout
+                self.ui_state_list = wb_list_panel_common.ListItemState( True )
+            all_debug_messages.append( 'list place_holder %s' % self.ui_state_list.place_holder )
 
         if self.ui_state_focus is None:
             if self.event_handler is None:
-                if wb_config.debug_selection: print 'Z: getUpdateUiState() event_handler is None set tree'
+                all_debug_messages.append( 'event_handler is None set tree' )
                 self.ui_state_focus = self.ui_state_tree
             elif self.event_handler.isTreeHandler():
-                if wb_config.debug_selection: print 'Z: getUpdateUiState() event_handler is Tree set tree'
+                all_debug_messages.append( 'event_handler is Tree set tree' )
                 self.ui_state_focus = self.ui_state_tree
             else:
-                if wb_config.debug_selection: print 'Z: getUpdateUiState() event_handler is List set list'
+                all_debug_messages.append( 'event_handler is List set list' )
                 self.ui_state_focus = self.ui_state_list
-            if wb_config.debug_selection: print 'Z: getUpdateUiState() focus need_checkout', self.ui_state_focus.need_checkout
+            all_debug_messages.append( 'focus place_holder %s' % self.ui_state_focus.place_holder )
+
+        if wb_config.debug_selection and len(all_debug_messages)>0:
+            print 'ZF: getUpdateUiState() ------------------------------'
+            for message in all_debug_messages:
+                print '    %s' %message
 
     def clearUpdateUiState( self ):
-        if wb_config.debug_selection: print 'Z: clearUpdateUiState()'
+        if wb_config.debug_selection: print 'ZF: clearUpdateUiState()'
         self.ui_state_tree = None
         self.ui_state_list = None
         self.ui_state_focus = None
 
-
 #--------------------------------------------------------------------------------
 class LogCtrlPanel(wx.Panel):
-    ''' LogCtrlPanel '''
     def __init__( self, app, parent ):
         wx.Panel.__init__(self, parent, -1)
 
@@ -1043,8 +1093,9 @@ class LogCtrlPanel(wx.Panel):
 
         # Redirect the console IO to this panel
         sys.stdin = file( wb_platform_specific.getNullDevice(), 'r' )
-        sys.stdout = self
-        sys.stderr = self
+        if self.app.isStdIoRedirect():
+            sys.stdout = self
+            sys.stderr = self
 
         # Redirect log to the Log panel
         log_handler = LogHandler( self.text_ctrl )
@@ -1052,8 +1103,13 @@ class LogCtrlPanel(wx.Panel):
 
         wx.EVT_SIZE( self, wb_exceptions.TryWrapper( self.app.log, self.OnSize ) )
 
+        wx.EVT_MENU( self, wb_ids.id_NextControl, self.app.eventWrapper( self.OnNextControl ) )
+
     #---------- Event handlers ------------------------------------------------------------
 
+    def OnNextControl( self, event ):
+        print 'LogCtrlPanel.OnNextControl',event
+
     def OnSize( self, event ):
         self.text_ctrl.SetWindowSize( self.GetSize() )
 
@@ -1133,7 +1189,7 @@ class StyledLogCtrl(wx.stc.StyledTextCtrl):
         self.SetMarginWidth(2, 0)
 
         self.StyleSetSpec( wx.stc.STC_STYLE_DEFAULT, 
-                "size:%d,face:%s,fore:#000000" % (point_size, face) )
+                "size:%d,face:%s,fore:#000000" % (wb_config.point_size, wb_config.face) )
 
         self.StyleSetSpec( self.style_normal,   "fore:#000000" )
         self.StyleSetSpec( self.style_error,    "fore:#DC143C" )    # Crimson
@@ -1142,6 +1198,14 @@ class StyledLogCtrl(wx.stc.StyledTextCtrl):
         self.StyleSetSpec( self.style_critical, "fore:#BA55D3" )    # Medium Orchid
         self.StyleSetSpec( self.style_debug,    "fore:#DC143C" )    # Crimson
 
+        wx.EVT_KEY_DOWN( self, self.OnKeyDown )
+
+    def OnKeyDown( self, event ):
+        if event.GetKeyCode() == wx.WXK_TAB:
+            self.app.frame.tree_panel.SetFocus()
+        else:
+            event.Skip()
+
     def SetWindowSize( self, size ):
         wx.stc.StyledTextCtrl.SetSize( self, size )
         self.EnsureCaretVisible()
