@@ -20,6 +20,11 @@ id_View_UncontrolledFiles = wx.NewId()
 id_View_Recursive = wx.NewId()
 id_ClearLog = wx.NewId()
 
+id_View_Diff_WbDiff = wx.NewId()
+id_View_Diff_ExtGuiDiff = wx.NewId()
+id_View_Diff_ExtTextDiff = wx.NewId()
+id_View_Diff_SvnDiff = wx.NewId()
+
 id_Project_Add = wx.NewId()
 id_Project_Update = wx.NewId()
 id_Project_Delete = wx.NewId()
@@ -27,8 +32,6 @@ id_Project_Delete = wx.NewId()
 id_Bookmark_Add = wx.NewId()
 id_Bookmark_Manage = wx.NewId()
 
-id_Return_Hotkey = wx.NewId()
-
 id_Command_Shell = wx.NewId()
 id_File_Browser = wx.NewId()
 
@@ -70,6 +73,7 @@ id_SP_Update = wx.NewId()
 
 id_SP_Report_Updates = wx.NewId()
 id_SP_Report_LocksWc = wx.NewId()
-id_SP_Report_LocksRepos = wx.NewId()
+id_SP_Report_LocksRepos = wx.NewId()#1
 id_SP_Report_BranchChanges = wx.NewId()
 
+id_NextControl = wx.NewId()
