@@ -25,6 +25,12 @@ else:
 def getPreferencesFilename():
     return os.path.join( getApplicationDir(), 'WorkBench.ini' )
 
+def getPreferencesFilename():
+    return os.path.join( getApplicationDir(), 'WorkBench.xml' )
+
+def getOldPreferencesFilename():
+    return os.path.join( getApplicationDir(), 'WorkBench.ini' )
+
 def getLogFilename():
     return os.path.join( getApplicationDir(), 'WorkBench.log' )
 
