@@ -1,7 +1,7 @@
 '''
 
  ====================================================================
- Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2007 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -12,19 +12,42 @@
     wb_preferences.py
 
 '''
+import pprint
+
 import os
-import wx
+import types
 import ConfigParser
+import UserDict
+
+import xml.parsers.expat
+import xml.dom.minidom
+
+import wx
+
 import wb_platform_specific
 import wb_source_control_providers
-import string
+
+
+new_save = True
+
+class ParseError(Exception):
+    def __init__( self, value ):
+        self.value = value
+
+    def __str__( self ):
+        return str(self.value)
+
+    def __repr__( self ):
+        return repr(self.value)
+
 
 class Preferences:
-    def __init__( self, app ):
+    def __init__( self, app, pref_filename, old_pref_filename ):
         self.app = app
-        self.pref_filename = wb_platform_specific.getPreferencesFilename()
+        self.pref_filename = pref_filename
+        self.old_pref_filename = old_pref_filename
 
-        self.app.log.info( 'Reading preferences from %s' % self.pref_filename )
+        self.pref_data = None
 
         # all the preference section handles get created here
         self.pref_handlers = {}
@@ -41,8 +64,11 @@ class Preferences:
         self.readPreferences()
 
     def readPreferences( self ):
-        self.pref_data = ConfigParser.RawConfigParser()
-        self.pref_data.read( self.pref_filename )
+        try:
+            self.pref_data = PreferenceData( self.app.log, self.pref_filename, self.old_pref_filename )
+        except ParseError, e:
+            self.app.log.error( str(e) )
+            return
 
         for handler in self.pref_handlers.values():
             if self.pref_data.has_section( handler.section_name ):
@@ -68,12 +94,189 @@ class Preferences:
                 self.pref_data.add_section( handler.section_name )
                 handler.writePreferences( self.pref_data )
 
-            self.pref_data.write( open( self.pref_filename, 'w' ) )
+            f = file( self.pref_filename, 'w' )
+            self.pref_data.write( f )
+            f.close()
             self.app.log.info( 'Wrote preferences to %s' % self.pref_filename )
 
         except IOError, e:
             self.app.log.error( 'write preferences: %s' % e )
 
+class PreferenceData:
+    def __init__( self, log, xml_pref_filename, ini_pref_filename ):
+        self.all_sections = {}
+
+        if os.path.exists( xml_pref_filename ):
+            log.info( 'Reading preferences from %s' % xml_pref_filename )
+            self.__readXml( xml_pref_filename )
+        else:
+            log.info( 'Reading preferences from %s' % ini_pref_filename )
+            self.__readIni( ini_pref_filename )
+
+    def __readXml( self, xml_pref_filename ):
+        try:
+            f = file( xml_pref_filename, 'r' )
+            text = f.read()
+            f.close()
+
+            dom = xml.dom.minidom.parseString( text )
+
+        except IOError, e:
+            raise ParseError( str(e) )
+
+        except xml.parsers.expat.ExpatError, e:
+            raise ParseError( str(e) )
+
+        prefs = dom.getElementsByTagName( 'workbench-preferences' )[0]
+
+        self.__parseXmlChildren( prefs, self.all_sections )
+
+    def __parseXmlChildren( self, parent, data_dict ):
+        for child in parent.childNodes:
+            if child.nodeType == xml.dom.minidom.Node.ELEMENT_NODE:
+
+                if self.__hasChildElements( child ):
+                    child_data_dict = {}
+                    if child.nodeName in data_dict:
+                        if type(data_dict[ child.nodeName ]) != types.ListType:
+                            data_dict[ child.nodeName ] = [data_dict[ child.nodeName], child_data_dict]
+                        else:
+                            data_dict[ child.nodeName ].append( child_data_dict )
+                    else:
+                        data_dict[ child.nodeName ] = child_data_dict
+
+                    self.__parseXmlChildren( child, child_data_dict )
+                else:
+                    data_dict[ child.nodeName ] = self.__getText( child )
+
+    def __hasChildElements( self, parent ):
+        for child in parent.childNodes:
+            if child.nodeType == xml.dom.minidom.Node.ELEMENT_NODE:
+                return True
+        return False
+
+    def __getText( self, parent ):
+        all_text = []
+
+        for child in parent.childNodes:
+            if child.nodeType == xml.dom.minidom.Node.TEXT_NODE:
+                all_text.append( child.nodeValue )
+
+        return ''.join( all_text )
+
+    def __readIni( self, pref_filename ):
+        pref_data = ConfigParser.RawConfigParser()
+        pref_data.read( pref_filename )
+
+        for section_name in pref_data.sections():
+            section_dict = {}
+            self.all_sections[ section_name ] = section_dict
+
+            for option_name in pref_data.options( section_name ):
+                option_name_parts = option_name.split('_')
+
+                if option_name_parts[-1][0] in '0123456789':
+                    option_name_prefix = '_'.join( option_name_parts[:-1] )
+                    option_name_index = int(option_name_parts[-1])
+
+                    section_dict.setdefault( option_name_index, {} )
+                    section_dict[ option_name_index ][ option_name_prefix ]  = pref_data.get( section_name, option_name )
+                else:
+                    section_dict[ option_name ] = pref_data.get( section_name, option_name )
+
+        for section_name, num_items_name, list_name in [
+                    ('Bookmarks','num_bookmarks','bookmark'),
+                    ('Projects' ,'num_projects','project')]:
+            if self.has_section( section_name ):
+                section = self.all_sections[ section_name ]
+                section_list = []
+                num_items = int( section[ num_items_name ] )
+                for index in range( 1, num_items+1 ):
+                    section_list.append( section.pop( index ) )
+                section[ list_name ] = section_list
+
+    def __getElem( self, element_path ):
+        node = self._dom
+        for element_name in element_path:
+            children = node.childNodes
+            node = None
+            for child in children:
+                if child.nodeType == xml.dom.minidom.Node.ELEMENT_NODE and child.nodeName == element_name:
+                    node = child
+                    break
+            if node is None:
+                break
+
+        return node
+
+    def __getAttr( self, element_path, attrib_name ):
+        element = self.getElement( element_path )
+        if element.hasAttributes() and element.attributes.has_key( attrib_name ):
+            return element.attributes[ attrib_name ].value
+        return default
+
+
+    def has_section( self, section_name ):
+        return section_name in self.all_sections
+
+    def len_section( self, section_name, option_name ):
+        if type(self.all_sections[ section_name ][ option_name ]) == types.ListType:
+            length = len( self.all_sections[ section_name ][ option_name ] )
+        else:
+            length = 1
+        return length
+
+    def has_option( self, section_name, option_name ):
+        return option_name in self.all_sections[ section_name ]
+
+    def get( self, section_name, option_name ):
+        return self.all_sections[ section_name ][ option_name ]
+
+    def getint( self, section_name, option_name ):
+        return int( self.get( section_name, option_name ).strip() )
+
+    def getboolean( self, section_name, option_name ):
+        return self.get( section_name, option_name ).strip().lower() == 'true'
+
+    def remove_section( self, section_name ):
+        if section_name in self.all_sections:
+            del self.all_sections[ section_name ]
+
+    def add_section( self, section_name ):
+        self.all_sections[ section_name ] = {}
+
+    def append_dict( self, section_name, list_name, data ):
+        item_list = self.all_sections[ section_name ].setdefault( list_name, [] )
+        item_list.append( data )
+
+    def set( self, section_name, option_name, value ):
+        self.all_sections[ section_name ][ option_name ] = value
+
+    def write( self, f ):
+        f.write( '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>\n' )
+        f.write( '<workbench-preferences>\n' )
+        self.__writeDictionary( f, self.all_sections, 4 )
+        f.write( '</workbench-preferences>\n' )
+
+    def __writeDictionary( self, f, d, indent ):
+        all_key_names = d.keys()
+        all_key_names.sort()
+
+        for key_name in all_key_names:
+            value = d[ key_name ]
+            if type(value) == types.DictType:
+                if len(value) > 0:
+                    f.write( '%*s<%s>\n' % (indent, '', key_name) )
+                    self.__writeDictionary( f, value, indent + 4 )
+                    f.write( '%*s</%s>\n' % (indent, '', key_name) )
+            elif type(value) == types.ListType:
+                for item in value:
+                    f.write( '%*s<%s>\n' % (indent, '', key_name) )
+                    self.__writeDictionary( f, item, indent + 4 )
+                    f.write( '%*s</%s>\n' % (indent, '', key_name) )
+            else:
+                f.write( '%*s<%s>%s</%s>\n' % (indent, '', key_name, unicode(value).encode('utf-8'), key_name) )
+
 class PreferenceSection:
     def __init__( self, section_name ):
         self.section_name = section_name
@@ -120,39 +323,36 @@ class SetOption:
         self.section_name = section_name
 
     def set( self, name, value, sep='' ):
-        if type(value) == type([]):
-            value = string.join( value, sep )
+        if type(value) == types.ListType:
+            value = sep.join( value )
 
         self.pref_data.set( self.section_name, name, value )
 
-class GetIndexedOption(GetOption):
-    def __init__( self, pref_data, section_name, index ):
-        GetOption.__init__( self, pref_data, section_name )
-        self.index = index + 1
+class GetIndexedOption:
+    def __init__( self, pref_data, section_name, index, index_name ):
+        self.pref_list = pref_data.get( section_name, index_name )
+        if type(self.pref_list) != types.ListType:
+            self.pref_list = [self.pref_list]
+
+        self.index = index
 
     def has( self, name ):
-        return self.pref_data.has_option( self.section_name, '%s_%d' % (name, self.index) )
+        return name in self.pref_list[ self.index ]
+
+    def get( self, name ):
+        return self.pref_list[ self.index ][ name ]
 
     def getstr( self, name ):
-        return self.pref_data.get( self.section_name, '%s_%d' % (name, self.index) ).strip()
+        return self.get( name ).strip()
 
     def getint( self, name ):
-        return self.pref_data.getint( self.section_name, '%s_%d' % (name, self.index) )
+        return int( self.getstr( name ) )
 
     def getfloat( self, name ):
-        return self.pref_data.getfloat( self.section_name, '%s_%d' % (name, self.index) )
+        return float( self.getstr( name ) )
 
     def getbool( self, name ):
-        return self.pref_data.getboolean( self.section_name, '%s_%d' % (name, self.index) )
-
-class SetIndexedOption:
-    def __init__( self, pref_data, section_name, index ):
-        self.pref_data = pref_data
-        self.section_name = section_name
-        self.index = index + 1
-
-    def set( self, name, value ):
-        self.pref_data.set( self.section_name, '%s_%d' % (name, self.index), value )
+        return self.getstr( name ) == 'true'
 
 class ProjectsPreferences(PreferenceSection):
     def __init__( self, app ):
@@ -165,9 +365,9 @@ class ProjectsPreferences(PreferenceSection):
         if not pref_data.has_section( self.section_name ):
             return
 
-        num_projects = pref_data.getint( self.section_name, 'num_projects' )
+        num_projects = pref_data.len_section( self.section_name, 'project' )
         for index in range( num_projects ):
-            get_option = GetIndexedOption( pref_data, self.section_name, index )
+            get_option = GetIndexedOption( pref_data, self.section_name, index, 'project' )
             provider = get_option.getstr( 'provider' )
 
             if wb_source_control_providers.hasProvider( provider ):
@@ -179,12 +379,15 @@ class ProjectsPreferences(PreferenceSection):
     def writePreferences( self, pref_data ):
         pref_data.remove_section( self.section_name )
         pref_data.add_section( self.section_name )
-        pref_data.set( self.section_name, 'num_projects', len( self.all_projects ) )
-        for index, pi in enumerate( self.all_projects.values() ):
-            pi.writePreferences( SetIndexedOption( pref_data, self.section_name, index ) )
+
+        for pi in self.all_projects.values():
+            pref_dict = {}
+
+            pi.writePreferences( pref_dict )
+            pref_data.append_dict( self.section_name, 'project', pref_dict )
 
     def _by_project_name( self, a, b ):
-        return cmp( a.project_name, b.project_name )
+        return cmp( a.project_name.lower(), b.project_name.lower() )
 
     def getProjectList( self ):
         pl = self.all_projects.values()
@@ -222,9 +425,9 @@ class BookmarksPreferences(PreferenceSection):
         if get_option.has( 'leaf_names_to_ignore' ):
             self.leaf_names_to_ignore = get_option.getstrlist( 'leaf_names_to_ignore', ',' )
 
-        num_bookmarks = get_option.getint( 'num_bookmarks' )
+        num_bookmarks = pref_data.len_section( self.section_name, 'bookmark' )
         for index in range( num_bookmarks ):
-            get_option = GetIndexedOption( pref_data, self.section_name, index )
+            get_option = GetIndexedOption( pref_data, self.section_name, index, 'bookmark' )
             bookmark_name = get_option.getstr( 'bookmark_name' )
 
             provider = get_option.getstr( 'provider' )
@@ -248,15 +451,15 @@ class BookmarksPreferences(PreferenceSection):
         pref_data.add_section( self.section_name )
 
         set_option = SetOption( pref_data, self.section_name )
-        #set_option.set( 'menu_style', self.menu_style )
-        #set_option.set( 'leaf_names_to_ignore', self.leaf_names_to_ignore, ',' )
 
-        set_option.set( 'num_bookmarks', len( self.all_bookmarks ) )
-        for index, bookmark_name in enumerate( self.all_bookmarks.keys() ):
-            set_option = SetIndexedOption( pref_data, self.section_name, index )
-            set_option.set( 'bookmark_name', bookmark_name )
+        for bookmark_name in self.all_bookmarks.keys():
+            pref_dict = {}
+            pref_dict[ 'bookmark_name' ] = bookmark_name
+
             pi = self.all_bookmarks[ bookmark_name ]
-            pi.writePreferences( set_option )
+            pi.writePreferences( pref_dict )
+
+            pref_data.append_dict( self.section_name, 'bookmark', pref_dict )
 
     def addBookmark( self, pi, name=None ):
         if name is None:
@@ -525,3 +728,26 @@ class DiffToolPreferences(PreferenceSection):
             set_option.set( 'diff_tool_options', self.gui_diff_tool_options )
         if self.shell_diff_tool_options != '':
             set_option.set( 'shell_diff_tool_options', self.shell_diff_tool_options )
+
+if __name__ == '__main__':
+    class FakeApp:
+        def __init__( self ):
+            self.log = self
+
+        def info( self, message ):
+            print 'Info:',message
+
+        def error( self, message ):
+            print 'Error:',message
+
+        def getCredentials( self ):
+            pass
+
+    import wb_subversion_provider
+
+    # Register all supported source control providers
+    wb_subversion_provider.registerProvider()
+
+    p = Preferences( FakeApp(), '/tmp/t.xml', '/tmp/t.ini' )
+    pprint.pprint( p.pref_data.all_sections )
+    p.writePreferences()
