@@ -184,7 +184,6 @@ class EditorPage(PagePanel):
 
         return True
 
-
 class DiffToolPage(PagePanel):
     def __init__( self, notebook, app ):
         self.app = app
@@ -192,7 +191,6 @@ class DiffToolPage(PagePanel):
 
     def initControls( self ):
         p = self.app.prefs.getDiffTool()
-
         
         self.mode_values = ['built-in', 'external-gui-diff', 'external-shell-diff', 'svn-diff']
         self.diff_tools = {'built-in': '', 'external-gui-diff': p.gui_diff_tool, 'external-shell-diff': p.shell_diff_tool, 'svn-diff': ''}
@@ -237,23 +235,28 @@ class DiffToolPage(PagePanel):
 
     def savePreferences( self ):
         self.OnModeChange( None )
+
         p = self.app.prefs.getDiffTool()
-        p.diff_tool_mode = self.mode
         p.gui_diff_tool = self.diff_tools['external-gui-diff']
         p.shell_diff_tool = self.diff_tools['external-shell-diff']
         p.gui_diff_tool_options = self.options['external-gui-diff']
         p.shell_diff_tool_options = self.options['external-shell-diff']
 
+        if p.diff_tool_mode == 'external-gui-diff' and p.gui_diff_tool == '':
+            p.diff_tool_mode = 'built-in'
+        if p.diff_tool_mode == 'external-shell-diff' and p.shell_diff_tool == '':
+            p.diff_tool_mode = 'built-in'
+
     def OnModeChange( self, event ):
-       self.diff_tools[self.mode] = self.text_ctrl_diff_tool.GetValue()
-       self.options[self.mode] = self.text_ctrl_options.GetValue()
-       self.mode = self.mode_values[self.mode_choice.GetSelection()]
-       self.text_ctrl_diff_tool.SetValue( self.diff_tools[self.mode] )
-       self.text_ctrl_options.SetValue( self.options[self.mode] )
-       external_command = self.mode in ('external-gui-diff', 'external-shell-diff')
-       self.text_ctrl_diff_tool.Enable( external_command )
-       self.browse_button.Enable( external_command )
-       self.text_ctrl_options.Enable( external_command )
+        self.diff_tools[self.mode] = self.text_ctrl_diff_tool.GetValue()
+        self.options[self.mode] = self.text_ctrl_options.GetValue()
+        self.mode = self.mode_values[self.mode_choice.GetSelection()]
+        self.text_ctrl_diff_tool.SetValue( self.diff_tools[self.mode] )
+        self.text_ctrl_options.SetValue( self.options[self.mode] )
+        external_command = self.mode in ('external-gui-diff', 'external-shell-diff')
+        self.text_ctrl_diff_tool.Enable( external_command )
+        self.browse_button.Enable( external_command )
+        self.text_ctrl_options.Enable( external_command )
 
     def OnBrowseExe( self, event ):
         if wx.Platform == '__WXMSW__':
