@@ -179,6 +179,8 @@ class CheckinFrame(wx.Frame):
         wx.EVT_MENU( self, id_exclude, self.app.eventWrapper( self.OnExcludeItem ) )
         wx.EVT_MENU( self, id_include, self.app.eventWrapper( self.OnIncludeItem ) )
 
+        self.log_message_ctrl.SetFocus()
+
     def OnExcludeItem( self, event ):
         self.list_handler.Cmd_Checkin_ExcludeItem( self.panel_list.getSelectedRows() )
         self.panel_list.drawList()
@@ -366,7 +368,6 @@ class CheckinListPanel(wb_list_panel_common.WbListPanelCommon):
                 (wx.ACCEL_ALT, ord('L'), wb_ids.id_SP_History),
                 (wx.ACCEL_ALT, ord('I'), wb_ids.id_SP_Info),
                 (wx.ACCEL_ALT, ord('P'), wb_ids.id_SP_Properties),
-                (wx.ACCEL_NORMAL, wx.WXK_RETURN, wb_ids.id_Return_Hotkey),
                 (wx.ACCEL_ALT, ord('O'), wb_ids.id_Shell_Open),
                 ]
         elif wx.Platform == '__WXMSW__':
@@ -376,7 +377,6 @@ class CheckinListPanel(wb_list_panel_common.WbListPanelCommon):
                 (wx.ACCEL_CTRL, ord('L'), wb_ids.id_SP_History),
                 (wx.ACCEL_CTRL, ord('I'), wb_ids.id_SP_Info),
                 (wx.ACCEL_CTRL, ord('P'), wb_ids.id_SP_Properties),
-                (wx.ACCEL_NORMAL, wx.WXK_RETURN, wb_ids.id_Return_Hotkey),
                 (wx.ACCEL_CTRL, ord('O'), wb_ids.id_Shell_Open),
                 ]
         else:
@@ -387,7 +387,6 @@ class CheckinListPanel(wb_list_panel_common.WbListPanelCommon):
                 (wx.ACCEL_CTRL, ord('L'), wb_ids.id_SP_History),
                 (wx.ACCEL_CTRL, ord('I'), wb_ids.id_SP_Info),
                 (wx.ACCEL_CTRL, ord('P'), wb_ids.id_SP_Properties),
-                (wx.ACCEL_NORMAL, wx.WXK_RETURN, wb_ids.id_Return_Hotkey),
                 ]
 
         return acc_init
