@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2007 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -268,18 +268,21 @@ def __getFileContents( project_info, path_info ):
         all_content_lines = wb_read_file.readFileContentsAsUnicode( path_info.path ).split('\n')
 
     elif path_info.revision.kind == pysvn.opt_revision_kind.base:
-        base_filename = os.path.join(
-            project_info.wc_path,
-            os.path.dirname( path_info.path ),
-            '.svn', 'text-base',
-            os.path.basename( path_info.path ) + '.svn-base' )
-        all_content_lines = wb_read_file.readFileContentsAsUnicode( base_filename ).split('\n')
+        all_content_lines = project_info.client_bg.cat(
+                url_or_path=path_info.path,
+                revision=path_info.revision ).split('\n')
 
     else:
-        all_content_lines = project_info.client_bg.cat(
-            url_or_path=path_info.path,
-            revision=path_info.revision,
-            peg_revision=path_info.peg_revision ).split('\n')
+        if path_info.peg_revision is not None:
+            all_content_lines = project_info.client_bg.cat(
+                url_or_path=path_info.peg_path,
+                revision=path_info.revision,
+                peg_revision=path_info.peg_revision ).split('\n')
+
+        else:
+            all_content_lines = project_info.client_bg.cat(
+                url_or_path=path_info.path,
+                revision=path_info.revision ).split('\n')
 
     return all_content_lines
 
@@ -293,27 +296,30 @@ def __getLocalFilename( app, project_info, path_info ):
 
     elif path_info.revision.kind == pysvn.opt_revision_kind.base:
         rev_description = 'BASE'
-        base_filename = os.path.join(
-            project_info.wc_path,
-            os.path.dirname( path_info.path ),
-            '.svn', 'text-base',
-            os.path.basename( path_info.path ) + '.svn-base' )
-        all_content = wb_read_file.readFileContentsAsUnicode( base_filename )
+        all_content = project_info.client_bg.cat(
+                url_or_path=path_info.path,
+                revision=path_info.revision )
 
     else:
         if path_info.revision.kind == pysvn.opt_revision_kind.head:
             rev_description = 'HEAD'
         else:
-           rev_description = 'R%d' % path_info.revision.number
+            rev_description = 'R%d' % path_info.revision.number
 
-        all_content = project_info.client_bg.cat(
-                url_or_path=path_info.peg_path,
-                peg_revision=path_info.peg_revision,
-                revision=path_info.revision )
+        if path_info.peg_revision is not None:
+            all_content = project_info.client_bg.cat(
+                    url_or_path=path_info.peg_path,
+                    peg_revision=path_info.peg_revision,
+                    revision=path_info.revision )
+        else:
+            all_content = project_info.client_bg.cat(
+                    url_or_path=path_info.path,
+                    revision=path_info.revision )
 
     # create a temp file with a name that is based on the original filename
-    prefix = '%s-%s-' % (os.path.basename( path_info.path ), rev_description)
-    fd, filename = tempfile.mkstemp( prefix=prefix, suffix='.tmp')
+    prefix = 'tmp-%s-%s-' % (os.path.basename( path_info.path ), rev_description)
+    suffix = os.path.splitext( path_info.path )[1]
+    fd, filename = tempfile.mkstemp( prefix=prefix, suffix=suffix )
     os.write( fd, all_content )
     os.close( fd )
 
