@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2007 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -23,6 +23,7 @@ import pysvn
 import wb_images
 import wb_subversion_utils
 import wb_subversion_diff
+import wb_config
 
 id_view_cmd = wx.NewId()
 id_diff_cmd = wx.NewId()
@@ -52,15 +53,41 @@ class LogEntry:
         self.message = message
         self.changed_paths = changed_paths
 
+        self.changed_paths.sort( self.by_changed_path )
+
+    def by_changed_path( self, a, b ):
+        return cmp( a.path, b.path )
+
+    def matchFilter( self, filter_field, filter_text ):
+        if filter_text == '':
+            return True
+
+        if self.label != '':
+            return True
+
+        if filter_field == 'Author':
+            return filter_text.lower() in self.author.lower()
+        elif filter_field == 'Comment':
+            return filter_text.lower() in self.message.lower()
+        elif filter_field == 'Path':
+            for changed_path in self.changed_paths:
+                if filter_text.lower() in changed_path.path.lower():
+                    return True
+
+            return False
+        else:
+            assert( False )
+            return False
+
 def getHistoryEntries( project_info, filename, limit, revision_end ):
-    history_entries = []
+    all_history_entries = []
     # need the URL and repos_root_URL
     # [0] first entry [0][1] the info
     info = project_info.client_bg.info2( filename, recurse=False )[0][1]
     if info.repos_root_URL is None:
         info = project_info.client_bg.info2( info.URL, recurse=False )[0][1]
 
-    log_entries = project_info.client_bg.log(
+    all_log_entries = project_info.client_bg.log(
                     filename,
                     strict_node_history=False,
                     discover_changed_paths=True,
@@ -68,9 +95,10 @@ def getHistoryEntries( project_info, filename, limit, revision_end ):
                     revision_end=revision_end )
 
     repos_path = info.URL[len(info.repos_root_URL):]
-    for log in log_entries:
-        history_entries.append(
-            LogEntry( log.revision.number,
+    for log in all_log_entries:
+        all_history_entries.append(
+            LogEntry(
+                log.revision.number,
                 info.repos_root_URL+repos_path,
                 log.author,
                 log.date,
@@ -85,19 +113,45 @@ def getHistoryEntries( project_info, filename, limit, revision_end ):
                         repos_path = changed_path.copyfrom_path
                     break
 
+    all_history_entries.sort( __cmpLogEntryHighToLow )
+
+    oldest_rev = log.revision.number
+
     tags_url = project_info.getTagsUrl( info.URL )
     if tags_url:
-        for ls_info in project_info.client_bg.ls( tags_url ):
-            history_entries.append( LogEntry(ls_info.created_rev.number,
-                '',
-                ls_info.last_author,
-                ls_info.time,
-                'Tag ' + ls_info.name.split('/')[-1],
-                '',
-                []) )
+        try:
+            for log in project_info.client_bg.log( tags_url, discover_changed_paths=True ):
+                for changed_path in log.changed_paths:
+                    if changed_path.copyfrom_revision is not None:
+                        # only include if it has taged an item in the history
+                        if changed_path.copyfrom_revision.number > oldest_rev:
+                            all_history_entries.append(
+                                LogEntry(
+                                    changed_path.copyfrom_revision.number,
+                                    __findTaggedUrl( all_history_entries, changed_path.copyfrom_revision.number ),
+                                    log.author,
+                                    log.date,
+                                    'Tag ' + changed_path.path.split('/')[-1],
+                                    log.message,
+                                    log.changed_paths) )
+                        break
 
-    history_entries.sort()
-    return info.URL, history_entries
+        except pysvn.ClientError, e:
+            self.log.info( 'Cannot find tags in %s - %s' % (tags_url, str(e)) )
+
+    all_history_entries.sort( __cmpLogEntryHighToLow )
+    return info.URL, all_history_entries
+
+def __cmpLogEntryHighToLow( a, b ):
+    return -cmp( a.rev_number, b.rev_number )
+
+def __findTaggedUrl( all_history_entries, tag_revnum ):
+    for entry in all_history_entries:
+        if entry.rev_number < tag_revnum:
+            return entry.url
+
+    # this cannot happen
+    raise RuntimeError( '__findTaggedUrl failed to find tagged url' )
 
 class HistoryDialog(wx.Dialog):
     def __init__( self, parent ):
@@ -208,10 +262,10 @@ class HistoryDialog(wx.Dialog):
             return pysvn.Revision( pysvn.opt_revision_kind.number, 0 )
 
 class HistoryFileFrame(wx.Frame):
-    def __init__( self, app, project_info, filename, url, log_entries ):
+    def __init__( self, app, project_info, filename, url, all_log_entries ):
         wx.Frame.__init__( self, None, -1, "History of %s" % filename, size=(700,500) )
 
-        self.panel = LogHistoryPanel( self, app, project_info, filename, url, log_entries )
+        self.panel = LogHistoryPanel( self, app, project_info, filename, url, all_log_entries )
 
         # Set the application icon
         self.SetIcon( wb_images.getIcon( 'wb.png' ) )
@@ -227,10 +281,10 @@ class HistoryFileFrame(wx.Frame):
                     info1, info2 )
 
 class HistoryDirFrame(wx.Frame):
-    def __init__( self, app, project_info, filename, url, log_entries ):
+    def __init__( self, app, project_info, filename, url, all_log_entries ):
         wx.Frame.__init__( self, None, -1, "History of %s" % filename, size=(700,500) )
 
-        self.panel = LogHistoryPanel( self, app, project_info, filename, url, log_entries )
+        self.panel = LogHistoryPanel( self, app, project_info, filename, url, all_log_entries )
 
         # Set the application icon
         self.SetIcon( wb_images.getIcon( 'wb.png' ) )
@@ -245,6 +299,110 @@ class HistoryDirFrame(wx.Frame):
                     app, project_info,
                     info1, info2 )
 
+
+class WbHistoryListCtrl(wx.ListCtrl):
+    def __init__( self, parent, log_history, list_id ):
+        wx.ListCtrl.__init__( self, parent, list_id,
+                                style=wx.LC_REPORT|wx.NO_BORDER|wx.LC_HRULES|wx.LC_VIRTUAL )
+        self.log_history = log_history
+
+    def OnGetItemText(self, item, col):
+        return self.log_history.OnGetItemText( item, col )
+
+    def OnGetItemImage(self, item):
+        return -1
+
+    def OnGetItemAttr(self, item):
+        return self.log_history.OnGetItemAttr( item )
+
+class PanelFilter(wx.Panel):
+    def __init__( self, parent, app, filter_field ):
+        wx.Panel.__init__(self, parent, -1)
+
+        self.app = app
+
+        self.background_colour = wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DFACE )
+
+        self.v_sizer = wx.BoxSizer( wx.VERTICAL )
+        self.h_sizer2 = wx.BoxSizer( wx.HORIZONTAL )
+
+        self.filter_changed_handler = None
+
+        self.filter_field_choices = ['Author','Comment','Path']
+        self.filter_choice_ctrl = wx.Choice( self, wx.NewId(), choices=self.filter_field_choices )
+        self.filter_choice_ctrl.SetSelection( self.filter_field_choices.index( filter_field ) )
+        if wx.Platform == '__WXMAC__':
+            self.filter_text_ctrl = wx.TextCtrl( self, wx.NewId(), '', size=(-1,-1) )
+        else:
+            self.filter_text_ctrl = wx.TextCtrl( self, wx.NewId(), '', size=(-1,10) )
+        self.filter_clear_button = wx.Button( self, wx.NewId(), 'X', style=wx.BU_EXACTFIT, size=(30, -1) )
+
+        border = 1
+        self.h_sizer2.Add( self.filter_choice_ctrl, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, border )
+        if wx.Platform == '__WXMAC__':
+            self.h_sizer2.Add( self.filter_text_ctrl, 1, wx.EXPAND|wx.ALL, border )
+        else:
+            self.h_sizer2.Add( self.filter_text_ctrl, 1, wx.EXPAND|wx.ALL, border+2 )
+        self.h_sizer2.Add( self.filter_clear_button, 0, wx.EXPAND|wx.LEFT|wx.RIGHT, border )
+
+        border = 3
+        self.v_sizer.Add( self.h_sizer2, 1, wx.EXPAND|wx.ALL, 0 )
+
+        wx.EVT_BUTTON( self, self.filter_clear_button.GetId(), self.OnClearFilterText )
+        wx.EVT_TEXT( self, self.filter_text_ctrl.GetId(), self.OnFilterTextChanged )
+        wx.EVT_CHOICE( self, self.filter_choice_ctrl.GetId(), self.OnFilterTypeChanged )
+
+        wx.EVT_KEY_DOWN( self, self.OnKeyDown )
+
+        self.SetAutoLayout( True )
+        self.SetSizer( self.v_sizer )
+        self.v_sizer.Fit( self )
+        self.Layout()
+
+    def OnKeyDown( self, event ):
+        if event.GetKeyCode() == wx.WXK_TAB:
+            self.app.frame.list_panel.SetFocus()
+        else:
+            event.Skip()
+
+    def setFocusFilter( self ):
+        self.filter_text_ctrl.SetFocus()
+
+    def setFilterChangedHandler( self, handler ):
+        self.filter_changed_handler = handler
+
+    def __callFilterChangedHandler( self ):
+        self.filter_changed_handler(
+            self.filter_field_choices[ self.filter_choice_ctrl.GetSelection() ],
+            self.filter_text_ctrl.GetValue() )
+
+    def updateHeader(self, url_name, path_name ):
+        if url_name is None:
+            url_name = ''
+        if path_name is None:
+            path_name = ''
+
+        self.url_text_ctrl.SetValue( url_name )
+        self.path_text_ctrl.SetValue( path_name )
+
+        self.SetBackgroundColour( self.background_colour )
+        self.Refresh()
+
+    def clearFilterText( self ):
+        self.filter_text_ctrl.Clear()
+        self.__callFilterChangedHandler()
+
+    def OnClearFilterText( self, event=None ):
+        self.filter_text_ctrl.Clear()
+        self.__callFilterChangedHandler()
+
+    def OnFilterTypeChanged( self, event ):
+        self.filter_text_ctrl.Clear()
+        self.__callFilterChangedHandler()
+
+    def OnFilterTextChanged( self, event ):
+        self.__callFilterChangedHandler()
+
 class LogHistoryPanel:
     col_revision = 0
     col_author = 1
@@ -257,13 +415,21 @@ class LogHistoryPanel:
     col_copyfrom_revision = 2
     col_copyfrom_path = 3
 
-    def __init__( self, parent, app, project_info, filename, url, log_entries ):
+    def __init__( self, parent, app, project_info, filename, url, all_log_entries ):
         self.parent = parent
         self.app = app
         self.project_info = project_info
         self.filename = filename
         self.url = url
-        self.log_entries = log_entries
+        self.all_log_entries = all_log_entries
+
+        # run from recent to old
+        self.all_log_entries.sort( self.by_rev )
+
+
+        self.filter_field = 'Comment'
+        self.filter_text = ''
+        self.all_filtered_log_entries = all_log_entries
 
         # Create the splitter windows
         self.splitter = wx.lib.splitter.MultiSplitterWindow( parent )
@@ -283,16 +449,17 @@ class LogHistoryPanel:
         self.splitter.AppendWindow( self.panel_comment, 100 )
         self.splitter.AppendWindow( self.panel_changed_paths, 150 )
 
-        # run from recent to old
-        self.log_entries.sort( self.cmpLogEntries )
-
         self.selected_revisions = {}
 
         self.v_sizer_history = wx.BoxSizer( wx.VERTICAL )
         self.v_sizer_comment = wx.BoxSizer( wx.VERTICAL )
         self.v_sizer_changed_paths = wx.BoxSizer( wx.VERTICAL )
 
-        self.list_ctrl = wx.ListCtrl( self.panel_history, id_list, wx.DefaultPosition, wx.DefaultSize, wx.LC_REPORT|wx.NO_BORDER)
+        self.panel_filter = PanelFilter( self.panel_history, app, self.filter_field )
+        self.panel_filter.setFilterChangedHandler( self.OnFilterChanged )
+
+        self.list_ctrl = WbHistoryListCtrl( self.panel_history, self, id_list )
+        self.all_item_attr = {}
 
         self.list_ctrl.InsertColumn( self.col_revision, "Revision" )
         self.list_ctrl.InsertColumn( self.col_author, "Author" )
@@ -330,17 +497,18 @@ class LogHistoryPanel:
 
         self.paths_label = wx.StaticText( self.panel_changed_paths, -1, 'Changed Paths' )
 
+        self.v_sizer_history.Add( self.panel_filter, 0, wx.EXPAND|wx.ALL, 0 )
         self.v_sizer_history.Add( self.list_ctrl, 2, wx.EXPAND|wx.ALL, 5 )
         self.v_sizer_comment.Add( self.comment_label, 0, wx.ALL, 5 )
         self.v_sizer_comment.Add( self.comment_ctrl, 2, wx.EXPAND|wx.ALL, 5 )
         self.v_sizer_changed_paths.Add( self.paths_label, 0, wx.ALL, 5 )
         self.v_sizer_changed_paths.Add( self.paths_ctrl, 2, wx.EXPAND|wx.ALL, 5 )
 
+        wx.EVT_LIST_ITEM_SELECTED( self.panel_history, id_list, self.OnListItemSelected )
+        wx.EVT_LIST_ITEM_DESELECTED( self.panel_history, id_list, self.OnListItemDeselected )
 
-        wx.EVT_LIST_ITEM_SELECTED( self.panel_history, id_list, self.OnListItemSelected)
-        wx.EVT_LIST_ITEM_DESELECTED( self.panel_history, id_list, self.OnListItemDeselected)
-        wx.EVT_LIST_ITEM_SELECTED( self.panel_history, id_paths, self.OnPathItemSelected)
-        wx.EVT_LIST_ITEM_DESELECTED( self.panel_history, id_paths, self.OnPathItemDeselected)
+        wx.EVT_LIST_ITEM_SELECTED( self.panel_history, id_paths, self.OnPathItemSelected )
+        wx.EVT_LIST_ITEM_DESELECTED( self.panel_history, id_paths, self.OnPathItemDeselected )
 
         self.initList()
 
@@ -358,9 +526,56 @@ class LogHistoryPanel:
                 sizer.Fit( panel )
                 panel.Layout()
 
-    def cmpLogEntries( self, a, b ):
-        return cmp( a.rev_number, b.rev_number )
+    # ----------------------------------------
+    def setFocusFilter( self ):
+        self.panel_filter.setFocusFilter()
 
+    def OnFilterChanged( self, field, text ):
+        self.filter_field = field
+        self.filter_text = text
+
+        if self.filter_text == '':
+            self.all_filtered_log_entries = self.all_log_entries
+        else:
+            self.all_filtered_log_entries = [
+                log_entry for log_entry in self.all_log_entries
+                    if log_entry.matchFilter( self.filter_field, self.filter_text )]
+
+        self.initList()
+
+    def OnGetItemText( self, index, col ):
+        log_entry = self.all_filtered_log_entries[ index ]
+        if col == self.col_revision:
+            return str(log_entry.rev_number)
+        elif col == self.col_author:
+            return log_entry.author
+        elif col ==  self.col_date:
+            return wb_subversion_utils.fmtDateTime( log_entry.date )
+        elif col ==  self.col_label:
+            return log_entry.label
+        elif col ==  self.col_message:
+            return log_entry.message.replace( '\n', ' ' )
+        else:
+            assert( False )
+
+    def OnGetItemImage( self, index ):
+        return -1
+
+    def OnGetItemAttr( self, index ):
+        log_entry = self.all_filtered_log_entries[ index ]
+        if len(log_entry.label) == 0:
+            colour = wb_config.colour_log_normal
+        else:
+            colour = wb_config.colour_log_tag
+
+        if colour not in self.all_item_attr:
+            attr = wx.ListItemAttr()
+            attr.SetTextColour( colour )
+            self.all_item_attr[ colour ] = attr
+
+        return self.all_item_attr[ colour ]
+
+    # ----------------------------------------
     def initButtons( self, sizer ):
         self.h_sizer = wx.BoxSizer( wx.HORIZONTAL )
 
@@ -379,20 +594,26 @@ class LogHistoryPanel:
         wx.EVT_BUTTON( self.panel_history, id_view_cmd, self.OnViewCommand )
         wx.EVT_BUTTON( self.panel_history, id_diff_cmd, self.OnDiffCommand )
 
+        wx.EVT_UPDATE_UI( self.panel_history, id_diff_cmd, self.OnUpdateUiDiffCommand )
+
+    def getSelectedRows( self ):
+        all_rows = []
+        item_index = -1
+        while True:
+            item_index = self.list_ctrl.GetNextItem( item_index, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED )
+            if item_index < 0:
+                break
+
+            all_rows.append( item_index )
+
+        return all_rows
+
     #---------- Event handlers ----------------------------------------------------------
     def OnListItemSelected( self, event):
         self.updateComment( event.m_itemIndex )
-        self.selected_revisions[ event.m_itemIndex ] = None
-        self.button_diff.Enable( len( self.selected_revisions ) in [1] )
-        self.button_diff.Enable( len( self.selected_revisions ) in [1,2] )
 
     def OnListItemDeselected( self, event):
         self.updateComment( -1 )
-        if event.m_itemIndex in self.selected_revisions:
-            del self.selected_revisions[ event.m_itemIndex ]
-
-        self.button_diff.Enable( len( self.selected_revisions ) in [1] )
-        self.button_diff.Enable( len( self.selected_revisions ) in [1,2] )
 
     def OnPathItemSelected( self, event):
         pass
@@ -403,8 +624,8 @@ class LogHistoryPanel:
     #---------- Comment handlers ------------------------------------------------------------
     def updateComment( self, index ):
         if index >= 0:
-            message = self.log_entries[ index ].message
-            all_paths_info = self.log_entries[ index ].changed_paths
+            message = self.all_filtered_log_entries[ index ].message
+            all_paths_info = self.all_filtered_log_entries[ index ].changed_paths
         else:
             message = ''
             all_paths_info = []
@@ -412,10 +633,7 @@ class LogHistoryPanel:
         self.comment_ctrl.SetValue( message )
         self.comment_ctrl.SetInsertionPoint( 0 )
 
-        self.log_entries.sort( self.by_rev )
-
         self.paths_ctrl.DeleteAllItems()
-        all_paths_info.sort( self.by_changed_path )
         for index, info in enumerate( all_paths_info ):
             self.paths_ctrl.InsertStringItem( index,
                 action_map.get( info.action, info.action ) )
@@ -442,42 +660,41 @@ class LogHistoryPanel:
         w,h = self.panel_changed_paths.GetClientSizeTuple()
         self.v_sizer_changed_paths.SetDimension( 0, 0, w, h )
 
-    def by_changed_path( self, a, b ):
-        return cmp( a.path, b.path )
-
     def by_rev( self, a, b ):
         # highest rev first
         return -cmp( a.rev_number, b.rev_number )
 
     def initList( self ):
-        self.log_entries.sort( self.by_rev )
-
-        for index, log_entry in enumerate( self.log_entries ):
-            self.list_ctrl.InsertStringItem( index, str(log_entry.rev_number) )
-            self.list_ctrl.SetStringItem( index, self.col_author, log_entry.author )
-            self.list_ctrl.SetStringItem( index, self.col_date, wb_subversion_utils.fmtDateTime( log_entry.date ) )
-            self.list_ctrl.SetStringItem( index, self.col_label, log_entry.label )
-            self.list_ctrl.SetStringItem( index, self.col_message, log_entry.message.replace( '\n', ' ' ) )
+        self.list_ctrl.DeleteAllItems()
+        self.list_ctrl.SetItemCount( len(self.all_filtered_log_entries) )
+        if len(self.all_filtered_log_entries) > 0:
+            self.list_ctrl.RefreshItems( 0, len(self.all_filtered_log_entries)-1 )
 
     #---------- Command handlers ----------------------------------------------------------
     def OnViewCommand( self, event ):
         print 'Log history View not implemented'
 
+    def OnUpdateUiDiffCommand( self, event ):
+        all_rows = self.getSelectedRows()
+        self.button_diff.Enable( len( all_rows ) in (1,2) )
+
     def OnDiffCommand( self, event ):
-        indices = self.selected_revisions.keys()
+        indices = self.getSelectedRows()
+
         indices.sort()
         indices.reverse()
 
         if len( indices ) not in (1,2):
             return
 
-
         info1 = wb_subversion_diff.PathInfoForDiff()
 
-        info1.path = self.log_entries[ indices[0] ].url
-        info1.revision = pysvn.Revision( pysvn.opt_revision_kind.number, self.log_entries[ indices[0] ].rev_number )
-        info1.peg_path = self.log_entries[ 0 ].url
-        info1.peg_revision = pysvn.Revision( pysvn.opt_revision_kind.number, self.log_entries[ 0 ].rev_number )
+        info1.path = self.all_filtered_log_entries[ indices[0] ].url
+        info1.revision = pysvn.Revision( pysvn.opt_revision_kind.number,
+                                self.all_filtered_log_entries[ indices[0] ].rev_number )
+        info1.peg_path = self.all_filtered_log_entries[ 0 ].url
+        info1.peg_revision = pysvn.Revision( pysvn.opt_revision_kind.number,
+                                self.all_filtered_log_entries[ 0 ].rev_number )
         info1.title = '%s@%d' % (info1.path, info1.revision.number)
 
         info2 = info1.copy()
@@ -489,8 +706,9 @@ class LogHistoryPanel:
             info2.peg_revision = info2.revision
             info2.title = self.filename
         else:
-            info2.path = self.log_entries[ indices[1] ].url
-            info2.revision = pysvn.Revision( pysvn.opt_revision_kind.number, self.log_entries[ indices[1] ].rev_number )
+            info2.path = self.all_filtered_log_entries[ indices[1] ].url
+            info2.revision = pysvn.Revision( pysvn.opt_revision_kind.number,
+                                self.all_filtered_log_entries[ indices[1] ].rev_number )
             info2.title = '%s@%d' % (info2.path, info2.revision.number)
 
         generator = self.parent.diffFunction(
