@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2007 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -26,6 +26,7 @@ import wb_read_file
 import wb_subversion_info_dialog
 import wb_subversion_properties_dialog
 import wb_subversion_diff
+import wb_config
 
 col_labels = [
         ('Name',        25, 10, 100, wx.LIST_FORMAT_LEFT),
@@ -236,10 +237,10 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
         if len(filter_text) > 0:
             if filter_field == 'Name':
                 self.all_files = [f for f in self.project_info.getFilesStatus()
-                            if filter_text in f.path[prefix_len:]]
+                            if filter_text.lower() in f.path[prefix_len:].lower()]
             elif filter_field == 'Author':
                 self.all_files = [f for f in self.project_info.getFilesStatus()
-                            if f.entry is not None and filter_text in f.entry.commit_author]
+                            if f.entry is not None and filter_text.lower() in f.entry.commit_author.lower()]
         else:
             self.all_files = self.project_info.getFilesStatus()
 
@@ -310,7 +311,6 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
         return self.column_info.getColumnOrder()[col]
 
     def OnGetItemText( self, index, col ):
-
         prefix_len = len( self.project_info.wc_path ) + 1
 
         column = self.column_info.getNameByColumn( col )
@@ -354,7 +354,7 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
 
     def OnGetItemAttr( self, index ):
         if self.project_info.need_checkout:
-            colour = wx.RED
+            colour = wb_config.colour_status_need_checkout
         else:
             colour = self.statusColour( self.all_files[ index ] )
 
@@ -453,6 +453,7 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
         state.need_checkin = True
         state.conflict = True
         state.file_exists = True
+        state.is_folder = True
 
         for row in all_rows:
             filename = self.all_files[ row ].path
@@ -464,6 +465,8 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
                 state.modified = False
                 state.conflict = False
                 state.file_exists = False
+            else:
+                state.is_folder = False
 
             text_status = self.getTextStatus( row )
             if text_status in [pysvn.wc_status_kind.unversioned]:
@@ -531,7 +534,8 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
         if status.entry is None:
             return ''
         else:
-            return os.path.join( self.project_info.wc_path, status.entry.conflict_old )
+            return os.path.join( self.project_info.wc_path,
+                    status.entry.conflict_old )
 
     def getConflictNew( self, row_or_status ):
         status = self.getStatusFromRowOrStatus( row_or_status )
@@ -561,29 +565,29 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
         return wb_subversion_utils._status_format( status )
 
     def getAllGreyFilenames( self ):
-        raise NotImplemented
+        raise NotImplementedError
 
     def statusColour( self, status ):
         # default colour when nothing special is know for the file
-        colour = wx.BLACK
+        colour = wb_config.colour_status_normal
 
         # show that a file is on the clipboard
         if status.path in self.getAllGreyFilenames():
-            colour = wx.Colour( 128, 128, 128 )
+            colour = wb_config.colour_status_disabled
 
         # show that a file is uncontrolled
         elif status.entry is None:
-            colour = wx.GREEN
+            colour = wb_config.colour_status_unversioned
 
         else:
             # show a file is locked
             if( status.is_locked ):
-                colour = wx.RED
+                colour = wb_config.colour_status_locked
             # show a file is modified
             elif( self.getTextStatus( status ) != pysvn.wc_status_kind.normal
             or self.getPropStatus( status ) not in [pysvn.wc_status_kind.normal,pysvn.wc_status_kind.none]
             or status.is_copied or status.is_switched ):
-                colour = wx.BLUE
+                colour = wb_config.colour_status_modified
 
         return colour
 
@@ -719,15 +723,15 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
                                     info1,
                                     info2 )
 
-            if type(generator) == types.GeneratorType:
-                while True:
-                    try:
-                        where_to_go_next = generator.next()
-                    except StopIteration:
-                        # no problem all done
-                        break
+                if type(generator) == types.GeneratorType:
+                    while True:
+                        try:
+                            where_to_go_next = generator.next()
+                        except StopIteration:
+                            # no problem all done
+                            break
 
-                    yield where_to_go_next
+                        yield where_to_go_next
 
         self.app.setAction( 'Ready' )
 
@@ -949,7 +953,8 @@ class SortList:
         __pychecker__ = '--no-returnvalues'
 
         if field == SubversionListHandlerCommon.col_name:
-            return status.path
+            return status.path.lower()
+
         if field == SubversionListHandlerCommon.col_state:
             # Use positive text_status first
             # then positive prop_status
