@@ -54,7 +54,7 @@ class ProjectInfo(wb_source_control_providers.ProjectInfo):
         self.tags_url = None
 
     def __repr__( self ):
-        return '<svn.ProjectInfo wc_path=%s>' % self.wc_path
+        return '<svn.ProjectInfo wc_path=%r>' % self.wc_path
         #return '<wb_subversion_provider.ProjectInfo url=%s, wc_path=%s>' % (self.url, self.wc_path)
 
     def init( self, project_name, **kws):
@@ -165,13 +165,13 @@ class ProjectInfo(wb_source_control_providers.ProjectInfo):
         wc_path = os.path.expanduser( wc_path )
         self.init( name, url=url, wc_path=wc_path, tags_url=tags_url )
 
-    def writePreferences( self, set_option ):
+    def writePreferences( self, pref_dict ):
         # save state into a preference file
-        wb_source_control_providers.ProjectInfo.writePreferences( self, set_option )
+        wb_source_control_providers.ProjectInfo.writePreferences( self, pref_dict )
 
-        set_option.set( 'url', self.url )
-        set_option.set( 'wc_path', self.wc_path )
-        set_option.set( 'tags_url', self.tags_url )
+        pref_dict[ 'url' ] = self.url
+        pref_dict[ 'wc_path' ] = self.wc_path
+        pref_dict[ 'tags_url' ] = self.tags_url
 
     def isEqual( self, pi ):
         return (self.provider_name == pi.provider_name
@@ -316,15 +316,20 @@ class ProjectInfo(wb_source_control_providers.ProjectInfo):
     def __proplist( self, path ):
         if os.path.isdir( path ):
             prop_file = os.path.join( path, '.svn', 'dir-props' )
+            base_prop_file = os.path.join( path, '.svn', 'dir-prop-base' )
         else:
             dirname, basename = os.path.split( path )
             prop_file = os.path.join( dirname, '.svn', 'props', basename + '.svn-work' )
+            base_prop_file = os.path.join( dirname, '.svn', 'prop-base', basename + '.svn-base' )
 
         result = {}
         try:
             f = file( prop_file )
         except EnvironmentError:
-            return result
+            try:
+                f = file( base_prop_file )
+            except EnvironmentError:
+                return result
 
         while True:
             line = f.readline()
