@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2006 Barry A Scott.  All rights reserved.
+ Copyright (c) 2006-2007 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -153,9 +153,9 @@ class ReportBranchChangesListHandler(wb_subversion_list_handler_common.Subversio
     def statusColour( self, file ):
         # show that a file is on the exclude list
         if file.path in self.getAllGreyFilenames():
-            return wx.Colour( 128, 128, 128 )
+            return wb_config.colour_status_disabled
         else:
-            return wx.BLACK
+            return wb_config.colour_status_normal
 
     def getContextMenu( self ):
         menu_template = \
