@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2007 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -32,6 +32,7 @@ import wb_subversion_report_branch_changes
 import wb_subversion_properties_dialog
 import wb_clipboard
 import wb_dialogs
+import wb_config
 
 class SubversionProject(wb_tree_panel.TreeProjectItem):
     def __init__( self, app, project_info ):
@@ -91,23 +92,23 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
             # no status available - make a guess
             if not os.path.exists( self.project_info.wc_path ):
                 # nothing there
-                return wx.RED
+                return wb_config.colour_status_need_checkout
             elif not os.path.exists( os.path.join( self.project_info.wc_path, '.svn' ) ):
                 # not versioned
-                return wx.GREEN
+                return wb_config.colour_status_unversioned
             else:
                 # versioned and present
-                return wx.BLACK
+                return wb_config.colour_status_normal
 
         elif not os.path.exists( dir_status.path ):
             # nothing there
-            return wx.RED
+            return wb_config.colour_status_need_checkout
         elif dir_status.text_status in [pysvn.wc_status_kind.unversioned]:
             # unversioned
-            return wx.GREEN
+            return wb_config.colour_status_unversioned
 
         # versioned and present
-        return wx.BLACK
+        return wb_config.colour_status_normal
     
     def getState( self ):
         state = wb_tree_panel.TreeState()
@@ -418,6 +419,8 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
         self.app.refreshFrame()
 
     def Cmd_Dir_Info( self ):
+        filename = self.project_info.wc_path
+
         try:
             if hasattr( self.project_info.client_fg, 'info2' ):
                 entry = self.project_info.client_fg.info2( filename, recurse=False )
@@ -427,7 +430,7 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
             dialog = wb_subversion_info_dialog.InfoDialog( self.app,
                     self.app.frame.tree_panel.tree_ctrl,
                     self.project_info.wc_path,
-                    entry[0] )
+                    entry )
             dialog.ShowModal()
 
         except pysvn.ClientError, e:
