@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2007 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -25,7 +25,10 @@ import wb_project_dialogs
 import wb_config
 
 class TreeState:
-    def __init__( self ):
+    def __init__( self, place_holder=False ):
+        # used to tell if the state reflects a items state or is just a place holder
+        self.place_holder = place_holder
+
         self.modified = False
         self.versioned = False
         self.new_versioned = False
@@ -34,6 +37,7 @@ class TreeState:
         self.need_checkout = False
         self.conflict = False
         self.file_exists = False
+        self.is_folder = True
         self.is_project_parent = False
 
     def printState( self, title='' ):
@@ -59,13 +63,18 @@ class WbTreePanel(wx.Panel):
         self.list_panel = frame.list_panel
         try_wrapper = wb_exceptions.TryWrapperFactory( self.app.log )
 
-        wx.Panel.__init__( self, parent, -1 )
+        wx.Panel.__init__( self, parent )
 
         self.tree_ctrl = WbTreeCtrl( self, self.app )
 
-        if wx.Platform == '__WXMAC__':
-            acc_mod = wx.ACCEL_ALT
+        if wb_config.focus_ring:
+            box = wx.BoxSizer()
+            box.Add( self.tree_ctrl, 1, wx.EXPAND|wx.ALL, 3)
+            self.SetSizer( box )
 
+            self.tree_ctrl.Bind( wx.EVT_PAINT, self.OnPaint )
+
+        if wx.Platform == '__WXMAC__':
             acc_init = [
                 (wx.ACCEL_ALT, ord('C'), wb_ids.id_SP_EditCopy),
                 (wx.ACCEL_ALT, ord('X'), wb_ids.id_SP_EditCut),
@@ -78,7 +87,8 @@ class WbTreePanel(wx.Panel):
                 (wx.ACCEL_ALT, ord('R'), wb_ids.id_SP_Revert),
                 (wx.ACCEL_ALT, ord('U'), wb_ids.id_SP_Update),
                 (wx.ACCEL_NORMAL, wx.WXK_DELETE, wb_ids.id_SP_Delete),
-                (wx.ACCEL_NORMAL, wx.WXK_RETURN, wb_ids.id_Return_Hotkey),
+                (wx.ACCEL_ALT, ord('O'), wb_ids.id_Shell_Open),
+                (wx.ACCEL_NORMAL, wx.WXK_TAB, wb_ids.id_NextControl),
                 ]
         elif wx.Platform == '__WXMSW__':
             acc_init = [
@@ -93,9 +103,9 @@ class WbTreePanel(wx.Panel):
                 (wx.ACCEL_CTRL, ord('R'), wb_ids.id_SP_Revert),
                 (wx.ACCEL_CTRL, ord('U'), wb_ids.id_SP_Update),
                 (wx.ACCEL_NORMAL, wx.WXK_DELETE, wb_ids.id_SP_Delete),
-                (wx.ACCEL_NORMAL, wx.WXK_RETURN, wb_ids.id_Return_Hotkey),
+                (wx.ACCEL_CTRL, ord('O'), wb_ids.id_Shell_Open),
+                (wx.ACCEL_NORMAL, wx.WXK_TAB, wb_ids.id_NextControl),
                 ]
-            acc_init.append( (wx.ACCEL_CTRL, ord('O'), wb_ids.id_Shell_Open) )
         else:
             # Unix
             acc_init = [
@@ -110,7 +120,8 @@ class WbTreePanel(wx.Panel):
                 (wx.ACCEL_CTRL, ord('R'), wb_ids.id_SP_Revert),
                 (wx.ACCEL_CTRL, ord('U'), wb_ids.id_SP_Update),
                 (wx.ACCEL_NORMAL, wx.WXK_DELETE, wb_ids.id_SP_Delete),
-                (wx.ACCEL_NORMAL, wx.WXK_RETURN, wb_ids.id_Return_Hotkey),
+                (wx.ACCEL_CTRL, ord('O'), wb_ids.id_Shell_Open),
+                (wx.ACCEL_NORMAL, wx.WXK_TAB, wb_ids.id_NextControl),
                 ]
 
         acc_tab = wx.AcceleratorTable( acc_init )
@@ -137,12 +148,28 @@ class WbTreePanel(wx.Panel):
         wx.EVT_SET_FOCUS( self.tree_ctrl, self.OnSetFocus )
         wx.EVT_KILL_FOCUS( self.tree_ctrl, self.OnKillFocus )
 
+        wx.EVT_MENU( self, wb_ids.id_NextControl, self.app.eventWrapper( self.OnNextControl ) )
+
         # start up by skipping ui updates until we have the tree control initialised
         self.__skip_update_ui = True
 
     def __repr__( self ):
         return '<WbTreePanel %r>' % self.getSelectionProjectInfo()
 
+    def OnPaint( self, event ):
+        dc = wx.PaintDC( self )
+        dc.Clear()
+        w, h = self.GetSize()
+        if self.FindFocus() == self.tree_ctrl:
+            print 'tree focus'
+            dc.SetPen( wx.Pen( "red", 1 ) )
+            dc.DrawRectangle( 0, 0, w, h )
+        else:
+            dc.SetPen( wx.Pen( "green", 1 ) )
+            dc.DrawRectangle( 0, 0, w, h )
+            print 'tree unfocus'
+        event.Skip()
+
     def initFrame( self ):
         bookmark_pi = None
 
@@ -180,13 +207,22 @@ class WbTreePanel(wx.Panel):
             self.gotoBookmark( self.last_position_bookmark_name )
 
     def OnSetFocus( self, event ):
-        if wb_config.debug_selection: print 'Z: WbTreePanel OnSetFocus'
+        if wb_config.debug_selection: print 'ZT: WbTreePanel OnSetFocus'
         self.frame.setEventHandler( self )
 
+        if wb_config.focus_ring:
+            self.Refresh()
+
+        event.Skip()
+
     def OnKillFocus( self, event ):
-        if wb_config.debug_selection: print 'Z: WbTreePanel OnKillFocus'
+        if wb_config.debug_selection: print 'ZT: WbTreePanel OnKillFocus'
         #self.frame.clearEventHandler()
-        pass
+
+        if wb_config.focus_ring:
+            self.Refresh()
+
+        event.Skip()
 
     def OnDragBegin( self, event ):
         #print 'WbTreePanel.OnDragBegin'
@@ -196,6 +232,9 @@ class WbTreePanel(wx.Panel):
         #print 'WbTreePanel.OnDragEnd'
         pass
 
+    def OnNextControl( self, event ):
+        self.frame.list_panel.setFocusFilter()
+
     def getSelectionProjectHandler( self ):
         item = self.tree_ctrl.GetSelection()
         if not item:
@@ -307,7 +346,7 @@ class WbTreePanel(wx.Panel):
                 child_item = self.tree_ctrl.AppendItem( this_item, project_info.project_name )
                 child_handler = provider.getProjectTreeItem( self.app, project_info )
                 self.tree_ctrl.SetPyData( child_item, child_handler )
-                self.tree_ctrl.SetItemTextColour( child_item, wx.BLUE )
+                self.tree_ctrl.SetItemTextColour( child_item, wb_config.colour_status_qqq )
 
         self.tree_ctrl.SortChildren( this_item )
 
@@ -384,6 +423,10 @@ class WbTreePanel(wx.Panel):
         self.list_panel.OnActivateApp( is_active )
 
     def refreshTree( self ):
+        # need to restore the tree event handler if its currently active
+        set_tree_handler = self.frame.isEventHandler( self )
+        self.app.log.debug( 'refreshTree set_tree_handler %r' % set_tree_handler )
+
         item = self.tree_ctrl.GetSelection()
         if not item:
             return
@@ -395,8 +438,13 @@ class WbTreePanel(wx.Panel):
 
         tree_pi.updateStatus()
         self.updateTreeItem( item )
+
         self.list_panel.drawList()
 
+        if set_tree_handler:
+            # restore handler
+            self.frame.setEventHandler( self )
+
     def expandSelectedTreeNode( self ):
         self.refreshTree()
         item = self.tree_ctrl.GetSelection()
@@ -428,7 +476,7 @@ class WbTreePanel(wx.Panel):
 
     # Handler for when a new selection is made in the tree control child
     def OnTreeSelChanged( self, event ):
-        if wb_config.debug_selection: print 'Z: WbTreePanel OnTreeSelChanged __skip_update_ui', self.__skip_update_ui
+        if wb_config.debug_selection: print 'ZT: WbTreePanel OnTreeSelChanged __skip_update_ui', self.__skip_update_ui
         if self.__skip_update_ui:
             return
 
@@ -436,7 +484,7 @@ class WbTreePanel(wx.Panel):
         self.changedSelection( tree_item )
 
     def changedSelection( self, tree_item ):
-        if wb_config.debug_selection: print 'Z: WbTreePanel changedSelection'
+        if wb_config.debug_selection: print 'ZT: WbTreePanel changedSelection'
         self.frame.clearUpdateUiState()
         self.frame.setEventHandler( self )
 
@@ -531,6 +579,9 @@ class WbTreePanel(wx.Panel):
         # move the selection to the clicked on node
         point = event.GetPosition();
         item, flags = self.tree_ctrl.HitTest( point )
+        if not item:
+            return
+
         self.tree_ctrl.SelectItem( item )
         self.changedSelection( self.tree_ctrl.GetPyData( item ) )
 
@@ -899,6 +950,22 @@ class WbTreeCtrl(wx.TreeCtrl):
         self.app.log.debug("GetItemByName failed to find item")
         return None
 
+    def OnCompareItems( self, a_item, b_item ):
+        # sort case blind
+
+        a_handler = self.GetPyData( a_item )
+        a_pi = a_handler.getProjectInfo()
+
+        b_handler = self.GetPyData( b_item )
+        b_pi = b_handler.getProjectInfo()
+
+        if a_handler.isProjectParent():
+            # compare nodes in the root - projects
+            cmp( self.GetItemText( a_item ).lower(), self.GetItemText( b_item ).lower() )
+        else:
+            # compare children of projects
+            return cmp( a_pi.wc_path.lower(), b_pi.wc_path.lower() )
+
 #--------------------------------------------------------------------------------
 #
 #
